/**
 * @brief Traitement simple d'images bmp
 * @author Ph Lefebvre
 * @arch Linux IA32 / 32 bits
 * @file ex81.c
 * 
*/

#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

typedef struct sCouleur {
	unsigned char r ; 
	unsigned char g ;
	unsigned char b ;
} PixelCouleur ;

char headBMP [54] ;

typedef struct simage {
	int la ;
	int ha ;
	unsigned char **r ; // coordonnées 0,0 -> en bas à gauche
	unsigned char **g ;
	unsigned char **b ;
} Image ;

/** Affiche une image 
@param nom : nom du fichier
*/
void affiImage (char *nom) {
	char commande [100] ;
	sprintf(commande, "/usr/bin/eog %s &", nom) ;
	system(commande);
}

/** Réserve en mémoire la place pour une nouvelle image
@param l : largeur de l'image
@param h : hauteur de l'image
@return une structure de données contenant l'image dont tous les points sont noirs
*/

Image newImage (int l, int h) {
	Image im ;
	int i ;
	im.la = l;
	im.ha = h ;
	im.r = (unsigned char**) calloc (im.la, sizeof(char*));
    for (i=0; i<im.la; i++ )
        im.r[i] = (unsigned char*) calloc (im.ha, sizeof(char));
        
    im.g = (unsigned char**) calloc (im.la, sizeof(char*));
    for (i=0; i<im.la; i++ )
        im.g[i] = (unsigned char*) calloc (im.ha, sizeof(char));
        
    im.b = (unsigned char**) calloc (im.la, sizeof(char*));
    for (i=0; i<im.la; i++ )
        im.b[i] = (unsigned char*) calloc (im.ha, sizeof(char));
    return (im) ;
}

/** Charge une image
@param fichier : nom du fichier
@return une tructure contenant l'image
*/
Image lireImageBMP(char *fichier) {
	int offsetFic, i, j ;
	int la, ha ;
	FILE *fp ;
	Image im ;
	
	affiImage( fichier) ;
	
	fp = fopen( fichier, "r") ;
	if (fp == NULL) {
		perror("Pb ouverture fichier");
		exit(0) ;
	}
	fread(headBMP, sizeof (char) ,54, fp ) ;  // Entete
	fseek(fp, 10, SEEK_SET ); // Lecture taille entete
	fread(&offsetFic, sizeof (int) ,1, fp ) ;
	if (offsetFic!=54) {
		printf("offset = %d -- PB format BMP\n", offsetFic ) ;
		fclose(fp) ;
		exit(0) ;
	}
	
	fseek(fp, 18, SEEK_SET ); // positionnement sur le champ largeur image
	fread(&la, sizeof (int) ,1, fp ) ; // Lecture largeur image
	fread(&ha, sizeof (int) ,1, fp ) ; // Lecture hauteur image
	
	im= newImage(la, ha) ;
	
	printf(" im.la= %d\n", im.la ) ;
	printf(" im.ha= %d\n", im.ha ) ;
	
	fseek(fp, 54, SEEK_SET ); // Lecture des pixels
	
	
	for (j=0; j<im.ha; j++ )
		for (i=0; i<im.la; i++ ) {
			fread(&(im.b[i][j]), sizeof (char) ,1, fp ); // ordre B, G, R
			fread(&(im.g[i][j]), sizeof (char) ,1, fp );
			fread(&(im.r[i][j]), sizeof (char) ,1, fp );
		}

	fclose(fp) ;
	return (im) ;
	
}

/** Sauvegarde une image sur disque
@param nomFic : nom du fichier
@param im : structure image
*/
void saveImageBMP(Image im, char * nomFic) {
	int  i, j ;
	FILE *fp ;

	fp = fopen( nomFic, "w") ;
	if (fp == NULL) {
		perror("Pb ouverture fichier");
		exit(0) ;
	}
	
	fwrite (headBMP, sizeof (char) ,54, fp ) ;  // Entete
	
	for (j=0; j<im.ha; j++ )
		for (i=0; i<im.la; i++ ) {
			fwrite(&(im.b[i][j]), sizeof (char) ,1, fp ); // ordre B, G, R
			fwrite(&(im.g[i][j]), sizeof (char) ,1, fp );
			fwrite(&(im.r[i][j]), sizeof (char) ,1, fp );
		}
	
	fclose(fp) ;
}

// main de test
void main() {
	Image im1, im2 ;
	
	im1 = lireImageBMP("chat.bmp");
	//im2 = filtreNB (im1) ; // A faire !!
	saveImageBMP(im1, "test2.bmp") ;
	affiImage("test2.bmp");
}

		
